@@ -38,16 +38,16 @@ static void		pf_read_inode_dirs(prefetch_args_t *, xfs_buf_t *);
 /* buffer priorities for the libxfs cache */
 
 #define B_DIR_BMAP	15
-#define B_DIR_META_2	13	/* metadata in secondary queue */
-#define B_DIR_META_H	11	/* metadata fetched for PF_META_ONLY */
-#define B_DIR_META_S	9	/* single block of metadata */
-#define B_DIR_META	7
-#define B_DIR_INODE	6
-#define B_BMAP		5
-#define B_INODE		4
+#define B_DIR_META_2	14	/* metadata in secondary queue */
+#define B_DIR_META_H	13	/* metadata fetched for PF_META_ONLY */
+#define B_DIR_META_S	12	/* single block of metadata */
+#define B_DIR_META	11
+#define B_DIR_INODE	10
+#define B_BMAP		9
+#define B_INODE		8
 
-#define B_IS_INODE(b)	(((b) & 1) == 0)
-#define B_IS_META(b)	(((b) & 1) != 0)
+#define B_IS_INODE(f)	(((f) & 5) == 0)
+#define B_IS_META(f)	(((f) & 5) != 0)
 
 #define DEF_BATCH_BYTES	0x10000
 
