@@ -17,6 +17,7 @@
  */
 
 #include <xfs/libxlog.h>
+#include <sys/resource.h>
 #include "avl.h"
 #include "avl64.h"
 #include "globals.h"
@@ -47,7 +48,7 @@ extern void	incore_init(xfs_mount_t *);
  */
 
 /*
- * -o (user-supplied override options)
+ * -o: user-supplied override options
  */
 
 char *o_opts[] = {
@@ -64,15 +65,41 @@ char *o_opts[] = {
 	NULL
 };
 
+/*
+ * -c: conversion options
+ */
+
+char *c_opts[] = {
+#define CONVERT_LAZY_COUNT	0
+	"lazycount",
+	NULL
+};
+
+
 static int	ihash_option_used;
 static int	bhash_option_used;
+static long	max_mem_specified;	/* in megabytes */
 
 static void
 usage(void)
 {
-	do_warn(
-_("Usage: %s [-nLvV] [-o subopt[=value]] [-l logdev] [-r rtdev] devname\n"),
-		progname);
+	do_warn(_(
+"Usage: %s [options] device\n"
+"\n"
+"Options:\n"
+"  -f           The device is a file\n"
+"  -L           Force log zeroing. Do this as a last resort.\n"
+"  -l logdev    Specifies the device where the external log resides.\n"
+"  -m maxmem    Maximum amount of memory to be used in megabytes.\n"
+"  -n           No modify mode, just checks the filesystem for damage.\n"
+"  -P           Disables prefetching.\n"
+"  -r rtdev     Specifies the device where the realtime section resides.\n"
+"  -v           Verbose output.\n"
+"  -c subopts   Change filesystem parameters - use xfs_admin.\n"
+"  -o subopts   Override default behaviour, refer to man page.\n"
+"  -t interval  Reporting interval in minutes.\n"
+"  -d           Repair dangerously.\n"
+"  -V           Reports version and exits.\n"), progname);
 	exit(1);
 }
 
@@ -87,7 +114,7 @@ err_string(int err_code)
 		err_message[XR_BAD_MAGIC] = _("bad magic number");
 		err_message[XR_BAD_BLOCKSIZE] = _("bad blocksize field");
 		err_message[XR_BAD_BLOCKLOG] = _("bad blocksize log field");
-		err_message[XR_BAD_VERSION] = _("bad version number");
+		err_message[XR_BAD_VERSION] = _("bad or unsupported version");
 		err_message[XR_BAD_INPROGRESS] =
 			_("filesystem mkfs-in-progress bit set");
 		err_message[XR_BAD_FS_SIZE_DATA] =
@@ -191,7 +218,7 @@ process_args(int argc, char **argv)
 	 * XXX have to add suboption processing here
 	 * attributes, quotas, nlinks, aligned_inos, sb_fbits
 	 */
-	while ((c = getopt(argc, argv, "o:fl:r:LnDvVdPMt:")) != EOF)  {
+	while ((c = getopt(argc, argv, "c:o:fl:m:r:LnDvVdPt:")) != EOF)  {
 		switch (c) {
 		case 'D':
 			dumpcore = 1;
@@ -222,6 +249,9 @@ process_args(int argc, char **argv)
 					ihash_option_used = 1;
 					break;
 				case BHASH_SIZE:
+					if (max_mem_specified)
+						do_abort(
+			_("-o bhash option cannot be used with -m option\n"));
 					libxfs_bhash_size = (int) strtol(val, 0, 0);
 					bhash_option_used = 1;
 					break;
@@ -234,6 +264,22 @@ process_args(int argc, char **argv)
 				}
 			}
 			break;
+		case 'c':
+			p = optarg;
+			while (*p) {
+				char *val;
+
+				switch (getsubopt(&p, (constpp)c_opts, &val)) {
+				case CONVERT_LAZY_COUNT:
+					lazy_count = (int)strtol(val, 0, 0);
+					convert_lazy_count = 1;
+					break;
+				default:
+					unknown('c', val);
+					break;
+				}
+			}
+			break;
 		case 'l':
 			log_name = optarg;
 			log_spec = 1;
@@ -245,6 +291,12 @@ process_args(int argc, char **argv)
 		case 'f':
 			isa_file = 1;
 			break;
+		case 'm':
+			if (bhash_option_used)
+				do_abort(_("-m option cannot be used with "
+						"-o bhash option\n"));
+			max_mem_specified = strtol(optarg, 0, 0);
+			break;
 		case 'L':
 			zap_log = 1;
 			break;
@@ -550,9 +602,10 @@ main(int argc, char **argv)
 	 * Calculations are done in kilobyte units.
 	 */
 
-	if (!bhash_option_used) {
+	if (!bhash_option_used || max_mem_specified) {
 		unsigned long 	mem_used;
-		unsigned long	phys_mem;
+		unsigned long	max_mem;
+		struct rlimit	rlim;
 
 		libxfs_icache_purge();
 		libxfs_bcache_purge();
@@ -560,27 +613,46 @@ main(int argc, char **argv)
 		cache_destroy(libxfs_bcache);
 
 		mem_used = (mp->m_sb.sb_icount >> (10 - 2)) +
-					(mp->m_sb.sb_dblocks >> (10 + 1));
-		phys_mem = libxfs_physmem() * 3 / 4;
+					(mp->m_sb.sb_dblocks >> (10 + 1)) +
+					50000;	/* rough estimate of 50MB overhead */
+		max_mem = max_mem_specified ? max_mem_specified * 1024 :
+						libxfs_physmem() * 3 / 4;
+
+		if (getrlimit(RLIMIT_AS, &rlim) != -1 &&
+					rlim.rlim_cur != RLIM_INFINITY) {
+			rlim.rlim_cur = rlim.rlim_max;
+			setrlimit(RLIMIT_AS, &rlim);
+			/* use approximately 80% of rlimit to avoid overrun */
+			max_mem = MIN(max_mem, rlim.rlim_cur / 1280);
+		} else
+			max_mem = MIN(max_mem, (LONG_MAX >> 10) + 1);
 
 		if (verbose > 1)
-			do_log(_("        - icount = %llu, imem = %lu, "
-				"dblock = %llu, dmem = %lu\n"),
-				mp->m_sb.sb_icount, mp->m_sb.sb_icount >> (10 - 2),
-				mp->m_sb.sb_dblocks, mp->m_sb.sb_dblocks >> (10 + 1));
+			do_log(_("        - max_mem = %lu, icount = %llu, "
+				"imem = %llu, dblock = %llu, dmem = %llu\n"),
+				max_mem, mp->m_sb.sb_icount,
+				mp->m_sb.sb_icount >> (10 - 2),
+				mp->m_sb.sb_dblocks,
+				mp->m_sb.sb_dblocks >> (10 + 1));
 
-		if (phys_mem <= mem_used) {
+		if (max_mem <= mem_used) {
 			/*
 			 * Turn off prefetch and minimise libxfs cache if
 			 * physical memory is deemed insufficient
 			 */
+			if (max_mem_specified)
+				do_abort(_("Required memory for repair is "
+					"greater that the maximum specified "
+					"with the -m option. Please increase "
+					"it to at least %lu.\n"),
+					mem_used / 1024);
 			do_prefetch = 0;
 			libxfs_bhash_size = 64;
 		} else {
-			phys_mem -= mem_used;
-			if (phys_mem >= (1 << 30))
-				phys_mem = 1 << 30;
-			libxfs_bhash_size = phys_mem / (HASH_CACHE_RATIO *
+			max_mem -= mem_used;
+			if (max_mem >= (1 << 30))
+				max_mem = 1 << 30;
+			libxfs_bhash_size = max_mem / (HASH_CACHE_RATIO *
 					(mp->m_inode_cluster_size >> 10));
 			if (libxfs_bhash_size < 512)
 				libxfs_bhash_size = 512;
